@@ -1,21 +1,23 @@
-from contextlib import contextmanager
 import logging
 import socket
 import ssl
+from contextlib import contextmanager
 from time import time
 
-from .block import Block
-from .blockstreamprofileinfo import BlockStreamProfileInfo
-from .clientinfo import ClientInfo
-from .context import Context
 from . import defines
 from . import errors
+from .block import Block
+from .blockstreamprofileinfo import BlockStreamProfileInfo
+from .bufferedreader import BufferedSocketReader
+from .clientinfo import ClientInfo
+from .compression import get_compressor_cls
+from .context import Context
+from .log import log_block
 from .progress import Progress
 from .protocol import Compression, ClientPacketTypes, ServerPacketTypes
 from .queryprocessingstage import QueryProcessingStage
 from .reader import read_varint, read_binary_str
 from .readhelpers import read_exception
-from .compression import get_compressor_cls
 from .settings.writer import write_settings
 from .writer import write_varint, write_binary_str
 
@@ -35,17 +37,57 @@ class Packet(object):
 
 
 class ServerInfo(object):
-    def __init__(self, name, version_major, version_minor, revision, timezone):
+    def __init__(self, name, version_major, version_minor, version_patch,
+                 revision, timezone, display_name):
         self.name = name
         self.version_major = version_major
         self.version_minor = version_minor
+        self.version_patch = version_patch
         self.revision = revision
         self.timezone = timezone
+        self.display_name = display_name
 
         super(ServerInfo, self).__init__()
 
 
 class Connection(object):
+    """
+    Represents connection between client and ClickHouse server.
+
+    :param host: host with running ClickHouse server.
+    :param port: port ClickHouse server is bound to. Defaults to ``9000``.
+    :param database: database connect to. Defaults to ``'default'``.
+    :param user: database user. Defaults to ``'default'``.
+    :param password: user's password. Defaults to ``''`` (no password).
+    :param client_name: this name will appear in server logs.
+                        Defaults to ``'python-driver'``.
+    :param connect_timeout: timeout for establishing connection.
+                            Defaults to ``10`` seconds.
+    :param send_receive_timeout: timeout for sending and receiving data.
+                                 Defaults to ``300`` seconds.
+    :param sync_request_timeout: timeout for server ping.
+                                 Defaults to ``5`` seconds.
+    :param compress_block_size: size of compressed block to send.
+                                Defaults to ``1048576``.
+    :param compression: specifies whether or not use compression.
+                        Defaults to ``False``. Possible choices:
+
+                            * ``True`` is equivalent to ``'lz4'``.
+                            * ``'lz4'``.
+                            * ``'lz4hc'`` high-compression variant of
+                              ``'lz4'``.
+                            * ``'zstd'``.
+
+    :param secure: establish secure connection. Defaults to ``False``.
+    :param verify: specifies whether a certificate is required and whether it
+                   will be validated after connection.
+                   Defaults to ``True``.
+    :param ssl_version: see :func:`ssl.wrap_socket` docs.
+    :param ca_certs: see :func:`ssl.wrap_socket` docs.
+    :param ciphers: see :func:`ssl.wrap_socket` docs.
+
+    """
+
     def __init__(
             self, host, port=None,
             database='default', user='default', password='',
@@ -183,7 +225,7 @@ class Connection(object):
             # performance tweak
             self.socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
 
-            self.fin = self.socket.makefile('rb')
+            self.fin = BufferedSocketReader(self.socket, defines.BUFFER_SIZE)
             self.fout = self.socket.makefile('wb')
 
             self.send_hello()
@@ -217,15 +259,13 @@ class Connection(object):
         self.block_out = None
 
     def disconnect(self):
+        """
+        Closes connection between server and client.
+        Frees resources: e.g. closes socket.
+        """
+
         if self.connected:
             # Close file descriptors before socket closing.
-            if self.fin:
-                try:
-                    self.fin.close()
-
-                except socket.error as e:
-                    logger.warning('Error on in file close: %s', e)
-
             if self.fout:
                 try:
                     self.fout.close()
@@ -252,9 +292,11 @@ class Connection(object):
     def send_hello(self):
         write_varint(ClientPacketTypes.HELLO, self.fout)
         write_binary_str(self.client_name, self.fout)
-        write_varint(defines.DBMS_VERSION_MAJOR, self.fout)
-        write_varint(defines.DBMS_VERSION_MINOR, self.fout)
-        write_varint(defines.CLIENT_VERSION, self.fout)
+        write_varint(defines.CLIENT_VERSION_MAJOR, self.fout)
+        write_varint(defines.CLIENT_VERSION_MINOR, self.fout)
+        # NOTE For backward compatibility of the protocol,
+        # client cannot send its version_patch.
+        write_varint(defines.CLIENT_REVISION, self.fout)
         write_binary_str(self.database, self.fout)
         write_binary_str(self.user, self.fout)
         write_binary_str(self.password, self.fout)
@@ -275,15 +317,27 @@ class Connection(object):
                     defines.DBMS_MIN_REVISION_WITH_SERVER_TIMEZONE:
                 server_timezone = read_binary_str(self.fin)
 
+            server_display_name = ''
+            if server_revision >= \
+                    defines.DBMS_MIN_REVISION_WITH_SERVER_DISPLAY_NAME:
+                server_display_name = read_binary_str(self.fin)
+
+            server_version_patch = server_revision
+            if server_revision >= \
+                    defines.DBMS_MIN_REVISION_WITH_VERSION_PATCH:
+                server_version_patch = read_varint(self.fin)
+
             self.server_info = ServerInfo(
                 server_name, server_version_major, server_version_minor,
-                server_revision, server_timezone
+                server_version_patch, server_revision,
+                server_timezone, server_display_name
             )
             self.context.server_info = self.server_info
 
             logger.debug(
-                'Connected to %s server version %s.%s.%s', server_name,
-                server_version_major, server_version_minor, server_revision
+                'Connected to %s server version %s.%s.%s, revision: %s',
+                server_name, server_version_major, server_version_minor,
+                server_version_patch, server_revision
             )
 
         elif packet_type == ServerPacketTypes.EXCEPTION:
@@ -348,6 +402,10 @@ class Connection(object):
         elif packet_type == ServerPacketTypes.EXTREMES:
             packet.block = self.receive_data()
 
+        elif packet_type == ServerPacketTypes.LOG:
+            block = self.receive_data()
+            log_block(block)
+
         elif packet_type == ServerPacketTypes.END_OF_STREAM:
             pass
 
